@@ -130,6 +130,11 @@ class BrownieProvider:
             return ret
         return getattr(project.interface, eth_contract)
 
+    def deploy(self, eth_contract, init_params, from_, **kwargs):
+        factory = self.get_contract_factory(eth_contract)
+        kwargs["from"] = from_
+        return factory.deploy(*init_params, kwargs)
+
     def init_eth_wrapper(self, eth_wrapper, owner, init_params, kwargs):
         eth_wrapper.owner = self.address_book.get_account(owner)
         for library in eth_wrapper.libraries_required:
@@ -149,5 +154,5 @@ class BrownieProvider:
             eth_wrapper.contract = eth_contract.deploy({"from": eth_wrapper.owner})
             eth_wrapper.contract.initialize(*init_params, {"from": eth_wrapper.owner})
 
-    def build_contract(self, contract_address, contract_factory):
-        return Contract.from_abi(contract_factory.name, contract_address, contract_factory.abi)
+    def build_contract(self, contract_address, contract_factory, contract_name=None):
+        return Contract.from_abi(contract_name or "Contract", contract_address, contract_factory.abi)
