@@ -3,13 +3,12 @@ from abc import ABC, abstractmethod
 from contextlib import contextmanager
 from functools import partial
 from .wadray import Wad, Ray
-from Crypto.Hash import keccak
 from environs import Env
 
 env = Env()
 
 SKIP_PROXY = env.bool("SKIP_PROXY", False)
-DEFAULT_PROVIDER = env.str("DEFAULT_PROVIDER", "brownie")
+DEFAULT_PROVIDER = env.str("DEFAULT_PROVIDER", None)
 
 MAXUINT256 = 2**256 - 1
 
@@ -17,7 +16,14 @@ _providers = {}
 
 
 def get_provider(provider_key=None):
-    return _providers[provider_key or DEFAULT_PROVIDER]
+    global DEFAULT_PROVIDER
+    provider_key = provider_key or DEFAULT_PROVIDER
+    if provider_key is None:
+        if len(_providers) == 1:
+            provider_key = next(iter(_providers.keys()))
+        else:
+            raise RuntimeError("No provider installed or no default provider specified")
+    return _providers[provider_key]
 
 
 def register_provider(provider_key, provider):
@@ -132,8 +138,29 @@ class ETHCall(ABC):
             self._handle_exception(err)
         return self.unparse(wrapper, self.eth_return_type, ret_value)
 
+    @classmethod
+    def parse_args(cls, wrapper, eth_args, *args, **kwargs):
+        """Used as helper for parsing arguments, for example for constructor"""
+        call_args = []
+        msg_args = {}
+        for i, (arg_name, arg_type) in enumerate(eth_args):
+            if i < len(args):
+                arg_value = args[i]
+            elif arg_name in kwargs:
+                arg_value = kwargs[arg_name]
+            else:
+                raise TypeError(f"missing required argument: '{arg_name}'")
+            if arg_type == "msg.sender":
+                msg_args["from"] = cls.parse(wrapper, "address", arg_value)
+            elif arg_type == "msg.value":
+                msg_args["value"] = cls.parse(wrapper, "amount", arg_value)
+            else:
+                call_args.append(cls.parse(wrapper, arg_type, arg_value))
+        return call_args, msg_args
+
     @classmethod
     def _parse_keccak256(cls, value):
+        from Crypto.Hash import keccak  # To avoid import of wrappers breaks if keccak not installed
         if value.startswith("0x"):
             return value
         k = keccak.new(digest_bits=256)
@@ -154,9 +181,16 @@ class ETHCall(ABC):
 class ETHWrapper:
     proxy_kind = None
     libraries_required = []
+    constructor_args = None
 
     def __init__(self, owner="owner", *init_params, **kwargs):
         self.provider_key = kwargs.get("provider_key", None)
+        if self.constructor_args is not None:
+            init_params, transaction_kwargs = self.eth_call.parse_args(
+                self, self.constructor_args, *init_params, **kwargs
+            )
+            if transaction_kwargs:
+                kwargs.update(transaction_kwargs)
         self.provider.init_eth_wrapper(self, owner, init_params, kwargs)
         self._auto_from = self.owner
 
@@ -174,7 +208,7 @@ class ETHWrapper:
         provider = get_provider(provider_key)
         if isinstance(contract, str):  # It's an address
             contract_factory = provider.get_contract_factory(cls.eth_contract)
-            contract = provider.build_contract(contract, contract_factory)
+            contract = provider.build_contract(contract, contract_factory, cls.eth_contract)
         return cls.build_from_contract(contract, owner, provider_key)
 
     @classmethod
